// FBX-Exercise- 02 -Solved
//

#include <fbxsdk.h>

// MSVC header
#include <assert.h>

FbxString nodeTypeName( FbxNodeAttribute::EType attrType )
{

    switch (attrType)
    {
        case FbxNodeAttribute::eBoundary:
            return FbxString("eBoundary");
            break;
        case FbxNodeAttribute::eCachedEffect:
            return FbxString("eCachedEffect");
            break;
        case FbxNodeAttribute::eCamera:
            return FbxString("eCamera");
            break;
        case FbxNodeAttribute::eCameraStereo:
            return FbxString("eCameraStereo");
            break;
        case FbxNodeAttribute::eCameraSwitcher:
            return FbxString("eCameraSwitcher");
            break;
        case FbxNodeAttribute::eLight:
            return FbxString("eLight");
            break;
        case FbxNodeAttribute::eLine:
            return FbxString("eLine");
            break;
        case FbxNodeAttribute::eLODGroup:
            return FbxString("eLODGroup");
            break;
        case FbxNodeAttribute::eMarker:
            return FbxString("eMarker");
            break;
        case FbxNodeAttribute::eMesh:
            return FbxString("eMesh");
            break;
        case FbxNodeAttribute::eNull:
            return FbxString("eNull");
            break;
        case FbxNodeAttribute::eNurbs:
            return FbxString("eNurbs");
            break;
        case FbxNodeAttribute::eNurbsCurve:
            return FbxString("eNurbsCurve");
            break;
        case FbxNodeAttribute::eNurbsSurface:
            return FbxString("eNurbsSurface");
            break;
        case FbxNodeAttribute::eOpticalMarker:
            return FbxString("eOpticalMarker");
            break;
        case FbxNodeAttribute::eOpticalReference:
            return FbxString("eOpticalReference");
            break;
        case FbxNodeAttribute::ePatch:
            return FbxString("ePatch");
            break;
        case FbxNodeAttribute::eShape:
            return FbxString("eShape");
            break;
        case FbxNodeAttribute::eSkeleton:
            return FbxString("eSkeleton");
            break;
        case FbxNodeAttribute::eSubDiv:
            return FbxString("eSubDiv");
            break;
        case FbxNodeAttribute::eTrimNurbsSurface:
            return FbxString("eTrimNurbsSurface");
            break;
        case FbxNodeAttribute::eUnknown:
            return FbxString("eUnknown");
            break;
        default:
            return FbxString("TYPE NOT MATCHED");
            break;
    }
}
		

void traverseScene( FbxNode *root, int level = 0 )
{
	// Level indentation
	for( int i = 0; i < level; i++ )
	{
        if (i+1 == level)
		    FBXSDK_printf( "+-" );
        else
            FBXSDK_printf( "  " );
	}

	//	Print out the name/type of this node
	//
	FbxString nodeName = root->GetName();
	FbxNodeAttribute *nodeAttr = root->GetNodeAttribute();
	FbxString nodeType = (nodeAttr==NULL) ? FbxString("IS NULL") : nodeTypeName( nodeAttr->GetAttributeType() );

	FBXSDK_printf("Node %s (type %s)\n", nodeName.Buffer(), nodeType.Buffer());

	// Recurse on the node children
	int numKids = root->GetChildCount();
	for (int k = 0; k < numKids; k++)
        {
		FbxNode *child = root->GetChild(k);
		traverseScene(child, level+1);
	}
}

int main(int argc, char* argv[])
{

    if (argc != 2) 
        return -1;
    //
    //  1. Use some utility classes to start with. Need to get the file name
    // and make sure it exists
    //
	FbxString sceneName = argv[1];//"exercise2.fbx";
    
    if (!FbxFileUtils::Exist(sceneName.Buffer()))
        return -1;

	//
	//	2. Create the FBX SDK manager		
	//
	FbxManager *fbxManager = FbxManager::Create();


    //
	//	3. Create and initialize the importer
	//
	FbxImporter *fbxImporter = FbxImporter::Create( fbxManager, "" );
	assert( fbxImporter != NULL );
	fbxImporter->Initialize( sceneName );

    //
	//	4. Use the importer to print out the file version
    //
	int major, minor, rev;
	fbxImporter->GetFileVersion( major, minor, rev );
	FBXSDK_printf( "File version is %d.%d rev %d\n", major, minor, rev );

    //
	//	5. Create the scene object
	//
	FbxScene *scene = FbxScene::Create( fbxManager, sceneName );
	assert( scene != NULL );

    //
	//	6. Import the file into the scene
	//
	if( !fbxImporter->Import(scene) )
	{
		FBXSDK_printf( "Import failed.\n" );
		return -1;
	}

    //
	//	7. Clean up memory for the importer
	//
	fbxImporter->Destroy();

    //
	//	8. Print out the list (array) of animation takes in the scene
	//
    FbxArray<FbxString*> lAnimStackNameArray;
	scene->FillAnimStackNameArray( lAnimStackNameArray );
	int takeCount = lAnimStackNameArray.GetCount();
	for (int i = 0; i < takeCount; i++) 
	{
        FBXSDK_printf("AnimStack %d: %s\n", i+1, lAnimStackNameArray[i]->Buffer());
	}

    //
	//	9. Traverse the scene and print out the node names and types
	//
	FbxNode *node = scene->GetRootNode();
	traverseScene( node );

    //
	//	10. Clean up SDK memory by destroying the SDK manager
	//
	fbxManager->Destroy();
	fbxManager = NULL;
	
    return 0;
}

