// fbx-exercise-03.cpp : Used for unit 3,4,5
//

#include <fbxsdk.h>
#include <assert.h>

FbxManager *gFbxManager = NULL;

//
//	Routines to create and destroy a single SDK manager for the
//	program
//
FbxManager *globalSdkMgr()
{
	if( gFbxManager == NULL )
	{
		gFbxManager = FbxManager::Create();
		assert( gFbxManager != NULL );
	}
	return gFbxManager;
}

void globalMgrDestroy()
{
	gFbxManager->Destroy();
	gFbxManager = NULL;
}

// Create a directional camera with target
// 
FbxNode* CreateCamera(FbxScene* pScene, FbxVector4 vecFrom, FbxVector4 vecTo )
{
    // ...

    return NULL;
}

// Create a light.
//
FbxNode* CreateLight(FbxScene* pScene, FbxDouble3 position)
{
    // ...

    return NULL;
}

// Create user defined properties
//
void CreateUserProperties(FbxNode *pNode, FbxObject *pObj) 
{
    // ...
    // (Hint: Use the ConnectSrcObject API for 
    // connecting the properties to the object .)
}

void PrintLevel(int level, bool bPlus = false)
{
	// Level indentation
	for( int i = 0; i < level; i++ )
	{
        if ((i+1 == level) && (bPlus))
		    FBXSDK_printf( "+-" );
        else
            FBXSDK_printf( "  " );
	}
}

// Iterate the scene, and for each node and 
// object look at the connections
//
void GetConnections( FbxNode *currentNode, int level = 0 )
{

	//	Print out the node name
	//
    PrintLevel(level, true);
    FBXSDK_printf("Node %s \n", currentNode->GetName());

    FbxObject* currentObj = currentNode->GetNodeAttribute();

    // Get the properties "connected" to the object
    // ... 
    // (Hint: Use the GetDstProperty API to ghet connections to the properties
    // because we used SetSrcObject when creating them.)

	//	Get and Print out the Source Objects
	//  ...

	//	Get and Print out the Destination Objects
	// ...

	// Recurse on the node children
	int numKids = currentNode->GetChildCount();
	for (int k = 0; k < numKids; k++)
    {
		FbxNode *child = currentNode->GetChild(k);
		GetConnections(child, level+1);
	}
}

// UNIT 4 Exercise

// Create a cube
//
FbxNode *CreateCube()
{
    // ...
    return NULL;
}

// Create materials and assign to a cube
//
void MaterialCube(FbxNode* pCube, FbxScene* scene)
{   
    // ...
}


// Animate the cube along one axis
//
void AnimateCube(FbxNode* pCube, FbxScene* scene)
{
    // ...
}

// OPTIONAL: Create a sphere. (Helper function)
//
FbxNode* CreateNurbsSphere(FbxScene* pScene, char* pName)
{
    // ...
    return NULL;
}

// OPTIONAL: Map texture over sphere. (Helper function)
//
void MapSphereTexture(FbxScene* pScene, FbxNode* pNurbs)
{
    // ...
}

// OPTIONAL: Map material over sphere. (Helper function)
//
void MapSphereMaterial(FbxScene* pScene, FbxNode* pNurbs)
{
    // ... 
}

// OPTIONAL: Create and assign texture to sphere with proper UV mapping
//
void CreateTextureSphere(FbxScene *scene)
{
    FbxNode* lNurbs = CreateNurbsSphere(scene, "TestSphere");

    if (lNurbs)
    {
        MapSphereMaterial(scene, lNurbs);
        MapSphereTexture(scene, lNurbs);

        // Build the node tree.
        FbxNode* lRootNode = scene->GetRootNode();
        lRootNode->AddChild(lNurbs);
        lNurbs->LclTranslation.Set(FbxVector4(0.0, 0.0, 200.0));
    }
}

// Find the ASCII format so we can export to it.
//
int findAsciiFormat (FbxManager *mgr)
{
    // ...

    return -1;
    
}
// Export the scene (ideally to ASCII format)
//
void ExportFbxScene( FbxManager *mgr, FbxScene *scene, char *pFileName )
{
    // ...

    return;
}

int main(int /*argc*/, char* /*argv[]*/)
{
    // Lab for Unit 3
	//	create an SDK manager
	//
	FbxManager *fbxSdkManager = globalSdkMgr();

	//	create a FbxScene
	//
	FbxScene *scene = FbxScene::Create( fbxSdkManager, "ADN_Scene" );
    
    // Get the root node
    //
    FbxNode *root = scene->GetRootNode();

    // Add a Directional camera to the root node of the scene
    // It should be postioned at 850.0, 700.0, 200.0, and be directed at 0.0, 0.0, 0.0
    //
    FbxNode* nodeCamera = CreateCamera(scene, FbxVector4(850.0, 700.0, 200.0), FbxVector4(0.0, 0.0, 0.0));
    if (nodeCamera)
        root->AddChild(nodeCamera);

    // Add a light to the root node of the scene. Position it at 1000,1000,1000.
    //
    FbxNode* nodeLight = CreateLight(scene, FbxDouble3(1000, 1000, 1000));
    if (nodeLight)
        root->AddChild(nodeLight);
    
    // Create some user defined properties
    // Add these properties to the light node, 
    // and also reference them with a connection to the camera object
    //
    if (nodeLight && nodeCamera)
    {
        CreateUserProperties(nodeLight, nodeCamera->GetNodeAttribute());

        // Try to find the properties.
        // First on the light node.
        FbxProperty propFromLightNode = nodeLight->FindProperty("ADN_StringList", false);
        if (propFromLightNode.IsValid())
            FBXSDK_printf("\nFound Light Node Prop: %s", propFromLightNode.GetNameAsCStr());
        // Then on the Camera object (NodeAttribute base). Here it will fail because the property is
        // not directly contained by the object or it's node. Instead a connection reference is made to
        // it. It will be found through the property connections APIs, though.
        FbxProperty propFromCamera = nodeCamera->GetNodeAttribute()->FindProperty("ADN_StringList", false);
        if (propFromCamera.IsValid())
            FBXSDK_printf("\nFound Camera Prop: %s", propFromCamera.GetNameAsCStr());
    }

    // Now iterate the scene's nodes and associated objects and get the connection infomration.
    //
    FBXSDK_printf("\n\n\nObject Connections\n");
    GetConnections(root);

    // Exercise for Unit 4
	//	create a cube and add it to the scene
	//
	FbxNode *nodeCube = CreateCube();
	if (nodeCube)
        root->AddChild( nodeCube );

    // Add meterials to the cube. A different color material for each side.
    //
    if (nodeCube)
        MaterialCube(nodeCube, scene);
    
    // OPTIONAL: Create a sphere and add a texture to it.
    //
    CreateTextureSphere(scene);

    // Exercise for Unit 5
	//	animate the cube to move along one axis
	//
    if (nodeCube)
	    AnimateCube(nodeCube, scene);

	//	save the scene to ASCII FBX file
	//
	ExportFbxScene( fbxSdkManager, scene, "output.fbx" );

	//	clean up memory for the manager
	//
	globalMgrDestroy();

    // Add a puase for ease of watching outputs during test runs
    scanf("pause");

	return 0;
}

